#!/usr/bin/env python
# File that manages the main windows
# CellTracking_framework MW 2012
# GPLv3+

version = "v0.9"
release = version+".0a2"
print "Hello, you are using CellTracking, version %s, release %s" % (version, release)

# ==== Importations
from threading import Thread
import time, sys, Queue, os, imp
core_path = './modules/modules_core/'
ct_Movies = imp.load_source('ct_Movies', core_path + 'ct_Movies.py')

import gobject
gobject.threads_init()

# ==== Classes
def get_version() :
	return (version, release)
class Object :
	"""General object for communication
	
	.. note::
		This object is pretty useless as it is not instanced, don't use it ;)
	"""

class Communication :
	"""This class was supposed to be used by the different modules to communicate,
	for instance in registering instance...
	
	.. note::
		This is not the case for the moment, since complex reference passing is used.
		But this should be used in the future, especially to register the main classes such
		as :py:class:`ct_GTK.Display_m` or :py:class:`ct_OpenCV.DDisplay_m`"""
	def __init__(self) :
		self.terminate = False
		self.keys = "Hot keys: \n" \
			"\tESC - quit the program\n" \
			"\tShift Right - play/pause\n" \
			"\tm - Play video faster\n" \
			"\tl - Play video slower\n" \
			"\te - save the frame as reference\n" \
			"\tb - next reference frame\n" \
			"\ti - switch view\n" \
			"\tp - switch mode manual/automatic\n"#\
			#"\ts - show/hide points\n" \
			#"\tr - auto-initialize tracking\n" \
			#"\tc - delete all the points\n" \
			#"\tn - switch the \"night\" mode on/off\n" \
			#"To add/remove a feature point click it\n"
		
class Terminate(Exception):
	"""This class inherits from :class:`Exception` and is used to close properly the program,
	since this exception is catched in the main loop in order to shut correctly the threads."""
	pass

class Display(Thread) :
	"""Class that manages the PyGTK windows
	This class initiates the thread that runs PyGTK. It must be initialized after
	this classes :
	
	* :py:class:`Communication`
	* :py:class:`ct_Movies.Movies`
	* :py:class:`ct_OpenCV.DDisplay_m`
	
	
	:param queue: A class:`Queue.Queue` instance
	:type queue: class:`Queue.Queue`
	:param comm: A :py:class:`Communication` instance
	:type comm: :class:`Communication`
	:param ddisplay: A :py:class:`ct_Movies.Movies` instance
	:type ddisplay: :class:`ct_Movies.Movies`

		"""
	def __init__(self, queue, comm, movies, ddisplay) :
		"""Lets initialize everything"""
		Thread.__init__(self)		# Initilizes the thread
		self.queue = queue
		self.terminate = False		# The ending flag
		self.communication = comm	# The object that allows communication between threads
		self.movies = movies		# Class managing movies
		self.ddisplay_m = ddisplay	# Referencing OpenCV playback
		#self.daemon = True		# Is a daemon (hardly killed on quit)
		
		ct_GTK = imp.load_source('ct_GTK', core_path + 'ct_GTK.py')
		#import ct_GTK
		self.ct_GTK = ct_GTK

	def run(self) :
		"""The main loop, launches the thread"""
		print "Starting the Display"
		main = self.ct_GTK.Main(gobject, self.communication)		# initializes self.gtk, self.display_m
		main.movies = self.movies
		main.ddisplay_m = self.ddisplay_m
		main.start()
		
		while not self.terminate:
			self.queue.put(sys.exc_info())	# Sending any ending signal to the main
			time.sleep(2)
			
	def stop(self) :
		"""Sets the terminate flag to end, in order to end the thread cleanly"""
		self.terminate = True
		
class OpenCV(Thread) :
	"""Class that manages the OpenCV windows
	This class initiates the thread that runs a loop and calls the OpenCV drawing
	functions. It must be initialized after this class :
	
	* :py:class:`Communication`
	
	
	:param queue: A class:`Queue.Queue` instance
	:type queue: class:`Queue.Queue`
	:param comm: A :py:class:`Communication` instance
	:type comm: :class:`Communication`
	"""
	def __init__(self, queue, comm) :
		"""Lets initialize everything"""
		Thread.__init__(self)
		self.terminate = False
		self.queue = queue
		self.communication = comm
		#self.daemon = True
		
		ct_OpenCV = imp.load_source('ct_OpenCV', core_path + 'ct_OpenCV.py')
		#import ct_OpenCV
		self.ct_OpenCV = ct_OpenCV
		self = self.ct_OpenCV.init(self)		# instanciates self.cv, self.ddisplay_m
		
	def run(self) :
		"""Sets the terminate flag to end, in order to end the thread cleanly"""
		print "Starting the video playback"
		self.ct_OpenCV.main(self, self.cv, self.communication)
		
		while not self.terminate :
			self.queue.put(sys.exc_info())	# Sending any ending signal to the main
			time.sleep(2)
			
	def stop(self) :
		"""Sets the terminate flag to end, in order to end the thread cleanly"""
		self.terminate = True

if __name__ == "__main__" :
	"""The main loop"""
	
	# Initializing the classes (and starting the threads)
	queue = Queue.Queue()				# Queue
	communication = Communication()			# Communication object
	movies = ct_Movies.Movies()			# Class managing movies
	opencv = OpenCV(queue, communication)		# OpenCV class
	
	display = Display(queue, communication, movies, opencv.ddisplay_m)	# Display (GTK) class
	opencv.start()
	display.start()
	
	print communication.keys	# Keyboard help (not implemented for the moment)
	
	# Main loop
	while 1 :
		try :
			if communication.terminate :
				raise Terminate
				sys.exit()
			queue.get()
			time.sleep(3)
		except KeyboardInterrupt :
			print "\nCtrl-C pressed"
			raise Terminate
		except Terminate :
			print "Exiting"
			display.stop()
			opencv.stop()
			display._Thread__stop()
			opencv._Thread__stop()
			tmp_ended = './tmp/tmp/tmp.ended'
			open(tmp_ended, 'w').close()
			break
