#! /usr/bin/env python
# _*-coding:utf-8-*-
# Modifié par PPPP, GPLv3+
# Script qui gère les modifications video depuis une source
# AntsTracking v0.1.7/v0.1.7

import cv, os, numpy, time, random

# ==================== Functions
def difference(reference, frame) :
	"""Fonction qui réalise la différence absolue par rapport à une image de référence prédéfinie"""
	diff = cv.CreateImage(cv.GetSize(frame), cv.IPL_DEPTH_8U, 3)
	cv.AbsDiff(frame, reference , diff)		# Différence absolue
	return diff

def laplacien(brut) :
	"""Application du Laplacien (détection de contours)"""
	planes = [cv.CreateImage((brut.width, brut.height), 8, 1) for i in range(3)]
        laplace = cv.CreateImage((brut.width, brut.height), cv.IPL_DEPTH_16S, 1)
        colorlaplace = cv.CreateImage((brut.width, brut.height), 8, 3)
	cv.Split(brut, planes[0], planes[1], planes[2], None)
	for plane in planes:
                cv.Laplace(plane, laplace, 3)
                cv.ConvertScaleAbs(laplace, plane, 1, 0)
	cv.Merge(planes[0], planes[1], planes[2], None, colorlaplace)
	return colorlaplace

def get_surrounding(shapes, im, i, j) :
	size = cv.GetSize(im)
	l = []
	if j-1 >= 0 and im[i, j-1] != 0 and shapes[i, j-1] not in l:
		l.append(shapes[i, j-1])
	if j-1 >= 0 and i-1 >= 0 and im[i-1, j-1] != 0 and shapes[i-1, j-1] not in l:
		l.append(shapes[i-1, j-1])
	if i-1 >= 0 and im[i-1, j] != 0 and shapes[i-1, j] not in l :
		l.append(shapes[i-1,j])
	return l
	

def segment_connected(image,threshold) :
	"""Function that segments the binary image based on contour finding of connected regions
	input image = B&W
	output image = Color"""
	
	# Initialization
	color = cv.CreateImage(cv.GetSize(image), 8, 3)		# Create output image
	
	# Canny edge extraction
	cv.Canny(image, image, 255, 255, aperture_size=3) # Canny Edge extraction
	
	# Extracting contour
	out_l = []		# Output list of contours
	cont = cv.FindContours(image, cv.CreateMemStorage(), mode=cv.CV_RETR_CCOMP, method=cv.CV_CHAIN_APPROX_SIMPLE, offset=(0, 0))	# Init - Returns a cvSeq
	
	while cont[:] != [] :
		if cv.ContourArea(cont) >= threshold :		# Replace with proper value
			out_l.append(cont)	
		im_tmp = cv.CreateImage(cv.GetSize(image), 8, 1) # Temporary image (FindContours modifies the initial image)
		cv.Copy(image, im_tmp)				# Fill the temporary image
		cont = cv.FindContours(im_tmp, cv.CreateMemStorage(), mode=cv.CV_RETR_CCOMP, method=cv.CV_CHAIN_APPROX_SIMPLE, offset=(0, 0))	# Returns a cvSeq
		cv.FillPoly(image, [cont], cv.RGB(0, 0, 0), lineType=8, shift=0)	# delete the contour on the image
	for c in out_l :
		cv.DrawContours(color, c, cv.RGB(random.randint(0,255), random.randint(0,255), random.randint(0,255)), cv.RGB(0, 100, 0), 0, thickness=1, lineType=8, offset=(0, 0))
	
	return [color, out_l]

def pre_treatment(imgDiff) :
	"""First filters to be applied before the outliers/isolated zones removal
	input image = Color RGB
	output image = B&W"""
    	grey_in = cv.CreateImage(cv.GetSize(imgDiff), 8, 1)
    	grey_out = cv.CreateImage(cv.GetSize(imgDiff), 8, 1)	
    	cv.CvtColor(imgDiff, grey_in, cv.CV_RGB2GRAY)			# To N&B
    	cv.Canny(grey_in, grey_out, 255, 255, aperture_size=3) # Edge extraction
	return grey_out

def normalize(imgDiff, seuil, erosion) :
	"""Fonction qui applique des filtres de contrasten"""
	[avg,std] = cv.AvgSdv(imgDiff)			# Calculating average and STD
	avg = [100-avg[0], 100-avg[1], 100-avg[2]]	# Normalized value
	cv.AddS(imgDiff, avg, imgDiff)			# Normalizing
	#cv.ConvertScale(imgDiff, imgDiff, scale=1.5)	# Increasing contrast (hazardous)
    	grey = cv.CreateImage(cv.GetSize(imgDiff), 8, 1)	
    	cv.CvtColor(imgDiff, grey, cv.CV_BGR2GRAY)			# Vers N&B
	cv.Smooth(grey, grey)						# Flou
	cv.Smooth(grey, grey)
	cv.Smooth(grey, grey)
	
	#print [avg[0],std[0]]
	mode = 0
	if mode == 0 :	
		s = avg[0]+seuil/8*std[0]
		cv.Threshold(grey, grey, s, 255, cv.CV_THRESH_BINARY)	# Seuil (bon = 17, 255)
	elif mode == 1 :		# Basic thresholding
		cv.Threshold(grey, grey, seuil, 255, cv.CV_THRESH_BINARY)	# Save
	return (grey, imgDiff)
	
def reference_init(path) :
	"""Function that initializes the reference folder and images"""
	images_path = []
	if os.path.isdir(path) and os.listdir(path) != [] :
		images_path = os.listdir(path)		# Get list of images
		images_path.sort(myComp)
		image = cv.LoadImage(path+images_path[-1])
		still_to_init = 0
	else :
		image = None
		if not os.path.isdir(path) :
			os.mkdir(path)
		still_to_init = 1
	dico = {"reference" : image, 
		"path" : path,
		"images" : images_path, 
		"index" : -1, 
		"to_be_initiated" : still_to_init}
	return dico
	
def reference_init2(dico, im) :
	if dico["to_be_initiated"] :
		print "Creating new reference"
		out = cv.CreateImage(cv.GetSize(im), 8, 3)	# Create black image
		cv.SetZero(out)
		cv.SaveImage(dico["path"]+"reference0.png", out)
		dico["images"] = ["reference0.png"]
		dico["reference"] = out
		dico["to_be_initiated"] = 0
	return dico
	
def new_reference(dico , dico_tracking, frame) :
	"""Fonction qui renvoie l'image de référence, si elle a été modifiée et la change si nécessaire"""
    	if dico_tracking['save'] :					# If saving a new reference is needed
		# Getting a free index
		i = 0
		while "reference" + str(i) + ".png" in dico["images"] :
			i+=1
		name = dico["path"]+"reference"+str(i)+".png"
		cv.SaveImage(name , frame)
		dico["images"].append(name)	# Updating dict
		dico["reference"] = frame

    		dico_tracking['save'] = 0				# Reset saving counter
	return dico
	
def next_reference(dico, dico_tracking) :
    	if dico_tracking['next'] :
    		try :
    			dico['index'] += 1
    			dico['reference'] = cv.LoadImage(dico["path"]+dico["images"][dico["index"]])# Chargement de l'image de référence      
    		except :
    			dico['index'] = 0
    			dico['reference'] = cv.LoadImage(dico["path"]+dico["images"][0])# Chargement de l'image de référence  
    		dico_tracking['next'] = 0 
    	return dico
    
def double_init(dico) :
	"""Initializing dictionnary for the difference of two successive images"""
	dico["buffer"]={"buffer_i":[], "out_i":None, "last_i" : -1}
	return dico

def myComp (x,y):
    import re
    def getNum(str): return float(re.findall(r'\d+',str)[0])
    return cmp(getNum(x),getNum(y))

def double_diff(dico, im, i) :
	"""Function that averages two images, i is the index of the image"""
	n = 5 	# Number of images to add
	if dico['buffer']["last_i"] != i :
		dico['buffer']["last_i"] = i
		if len(dico["buffer"]["buffer_i"]) == n :
			dico["buffer"]["buffer_i"].pop(0)	# Swap images
		dico["buffer"]["buffer_i"].append(im)
	out = cv.CreateImage(cv.GetSize(im), 8, 3)
	for i in range(len(dico["buffer"]["buffer_i"])) :
		tmp = cv.CreateImage(cv.GetSize(im), 8, 3)
		cv.ConvertScale(dico["buffer"]["buffer_i"][i], tmp, scale=1.0/n)
		cv.Add(out, tmp, out)
	dico["buffer"]["out_i"] = out
	return dico
