import numpy as np

class Phase :
	def __init__(self, cell, movie) :
		self.type = 'interval'
		self.color = ['grey', 'red', 'green']
		self.require = None
		self.name = 'Is it a cell?'
		self.s_name = 'is_cel'
		self.labels = [('unk', 'Unknown'), ('nocel','No cell'), ('cel','Cell')]
		
		#self.phase = [0]*movie.get_nb_frames()
		self.phase_interval = []
		self.cell = cell
		self.movie = movie
		self.current_phase = {'index' : None,
								'begin' : None,
								'end' : None}
		self.initiated = False
		self.desc = 'Is cell module'
	def get_save_dict(self) :
		dic = {}
		dic['name'] = {'desc' : 'Name of the phase', 'value' : self.name}
		dic['s_name'] = {'desc' : 'Short name of the phase', 'value' : self.s_name}
		dic['desc'] = {'desc' : 'Description of the phase', 'value' : self.desc}
		
		dic['initiated'] = {'desc' : 'if the phase is initiated (difference between nocell and unknown)', 'value' : self.initiated}
		dic['phase_interval'] = {'desc' : 'interval where the phase is positive (there is a cell)', 'value' : self.phase_interval}
		return dic
	def set_save_dict(self, dic) :
		self.initiated = dic['initiated']['value']
		self.phase_interval = dic['phase_interval']['value']
	def get_panel(self, gtk) :
		"""Returns a panel to be integrated in the feature panel for instance"""
		self.initiated = True
		self.box = gtk.HBox()
		b_new = self._bouton(gtk, 'New', self._new_phase, {})
		self.box.pack_start(b_new, False, False)
		b_set_begin = self._bouton(gtk, 'Set begin', self._set_begin, {})
		self.box.pack_start(b_set_begin, False, False)
		b_set_end = self._bouton(gtk, 'Set end', self._set_end, {})
		self.box.pack_start(b_set_end, False, False)
		b_delete = self._bouton(gtk, 'Delete', self._delete_phase, {})
		self.box.pack_start(b_delete, False, False)
		
		sep1 = gtk.VSeparator()
		sep1.show()
		self.box.pack_start(sep1)
		self.info_box = gtk.HBox()
		l_from = gtk.Label('From')
		l_from.show()
		self.info_box.pack_start(l_from, False, False)
		self.e_from = gtk.Entry()
		self.e_from.show()
		self.e_from.set_sensitive(False)
		self.e_from.set_width_chars(4)
		self.info_box.pack_start(self.e_from, False, False)
		l_to = gtk.Label(' to ')
		l_to.show()
		self.info_box.pack_start(l_to, False, False)
		self.e_to = gtk.Entry()
		self.e_to.set_width_chars(4)
		self.e_to.show()
		self.e_to.set_sensitive(False)
		self.info_box.pack_start(self.e_to, False, False)
		self.info_box.show()
		self.box.pack_start(self.info_box)
		
		sep2 = gtk.VSeparator()
		sep2.show()
		self.box.pack_start(sep2)
		self.cb_period = gtk.combo_box_new_text()
		self._phase_list_update()		# Populating cb_period
		self.cb_period.connect('changed', self._change_period, {})
		self.cb_period.show()
		self.box.pack_start(self.cb_period,True, True)
		b_auto = gtk.CheckButton('new frame auto')
		self.auto_mode = False
		b_auto.set_active(False)
		b_auto.show()
		b_auto.connect("toggled", self._auto_new_frame)
		self.box.pack_start(b_auto, True, True)
		self._info_box_update()
		if self.current_phase['index'] != None :
			self.cb_period.set_active(self.current_phase['index'])
		return self.box
	
	def get_phase(self, i) :
		"""Returns a tuple (bool, shortname, name, color)"""
		if self.initiated :
			ph = 1
		else :
			ph = 0

		for interval in self.phase_interval :
			if i>=interval[0] and i<=interval[1] :
				ph = 2
				break
		#boolean = self.phase[i]
		return (ph, self.labels[ph][0], self.labels[ph][1], self.color[ph])
	def find_phase(self, i) :
		fc = self.cell.get_cell(channel='bf_out')		# Getting OpenCV image
		c = self.cell.get_cell_without_reference(fc)	# Formatted as a an array, you should exclude 0 values before processing 
		r = self.cell.get_cell_reference(fc)		# idem Gets it without reference
		
		#folder = './exports/H1BF-test_export/' + str(self.cell.get_index())+'/'
		#path = folder + 'image'+str(i) + '.png'
		#np.save(folder + 'image_c'+str(i) + '.txt', c[:,:,0])
		#np.save(folder + 'image_r'+str(i) + '.txt', r[:,:,0])
		
		is_c = self._is_cell(c,r)
		#self.phase[i] = is_c
		if is_c == 2 :
			"""This procedure assumes that the intervals are sorted"""
			intervals = self.phase_interval
			if intervals != [] :
				last_int = intervals[-1]
				if last_int[1] == i-1 :
					self.phase_interval[-1] = (last_int[0], i)
			else :
				self.phase_interval.append((i,i))
	def _label(self, begin, end) :
		return "Frames " + str(begin) + '-' + str(end)	
	def _info_box_update(self) :
		fr = ''
		to = ''
		if self.current_phase['end'] != None :
			to = str(self.current_phase['end'])
		if self.current_phase['begin'] != None :
			fr = str(self.current_phase['begin'])
		self.e_to.set_text(to)
		self.e_from.set_text(fr)
	
	def _auto_save(self) :
		if self.auto_mode :
			if self._is_phase_ok(self.current_phase) :# Save the frame if correct
				self._save_phase(self.current_phase)
				self._new_phase('','')
		else :
			pass
	def _auto_new_frame(self, w, data=None) :
		self.auto_mode = w.get_active()
	def _new_phase(self, w, data=None) :
		"""This function creates a new period if the previous is finished or reset it
		if it is not"""
		# A different behavior must be implemented when editing instead of creating ???
		if self.auto_mode or not self._is_phase_ok(self.current_phase) :	# Or replace it
			self._set_default_phase()
		else : #if self._is_phase_ok(self.current_phase) :	# Save it as a new one
			#### The phase list must be sorted, don't forget
			self._save_phase(self.current_phase)
			self._set_default_phase()		# Create a new one
		self._info_box_update()
		self.cb_period.set_active(-1)	# Start with nothing selected
	def _save_phase(self, ph) :
		if ph['index'] == None :
			self.phase_interval.append((ph['begin'], ph['end']))	# Add it to list
		else :
			self.phase_interval[ph['index']] = (ph['begin'], ph['end'])
		self._phase_list_update()
		# Sorting will be implemented later...
		# Create list with original index [begin_frame, label, index]
		# Sort according to begin frame
		# Create correspondence list
	def _phase_list_update(self) :
		m = self.cb_period.get_model()	# Clean list
		for i in  m :
			self.cb_period.remove_text(0)
		for i in self.phase_interval :
			self.cb_period.append_text(self._label(i[0],i[1]))
		
	def _is_phase_ok(self, ph) :
		ret = True
		if ph['begin']==None or ph['end'] == None :
			ret = False
		if ph['begin'] > ph['end'] or ph['begin'] < 0 :
			ret = False
		return ret
	def _set_default_phase(self) :
		self.current_phase = {'index' : None,
								'begin' : None,
								'end' : None}
	def _set_begin(self, w, data=None) :
		i = self.movie.get_frame_index()
		self.current_phase['begin'] = i
		if self.current_phase['end'] != None and self.current_phase['end'] < i :
			n = self.movie.get_nb_frames()
			self.current_phase['end'] = min(i, n)
		self._auto_save()
		self._info_box_update()
		
	def _set_end(self, w, data=None) :
		i = self.movie.get_frame_index()
		self.current_phase['end'] = i
		if self.current_phase['begin'] != None and self.current_phase['begin'] > i :
			self.current_phase['begin'] = max(i,0)
		self._auto_save()
		self._info_box_update()
	def _delete_phase(self, w, data=None) :
		i = self.cb_period.get_active()
		if i != -1 :
			self.phase_interval.pop(i)
		self._info_box_update()
		self._phase_list_update()
		self._new_phase('','')
	def _change_period(self, w, data=None) :
		"""Selects a new period"""
		i = w.get_active()
		if i != -1 :
			ph = self.phase_interval[i]
			self.current_phase = {'begin' : ph[0], 'end' : ph[1], 'index' : i}
			self._info_box_update()
	def _is_cell(self, cell, ref) :
		ret = 1		# The return index
		threshold = 1.5
		cell = cell[:,:,0]
		c = cell[(cell!=255)&(cell!=0)]		# Filtering non saturated pixels
		ref = ref[3:-3,3:-3,0]
		r = ref[(ref!=255)&(ref!=0)]		# Filtering non saturated pixels
							# Excluding possibly misinterpolated pixels because of rotation
		ratio = np.std(c)/np.std(r)
		if ratio > threshold :
			ret = 2
		return ret
			
	def get_name(self) :
		return self.name
	def get_s_name(self) :
		return self.s_name
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	 # On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()# This packs the button into the window (a GTK container).
		return bouton
		
